#! /usr/bin/env perl
#
#	Specify two 'Command' methods to "build" (pseudo-install) files
#	in the local directory, one that executes a two command-list
#	and one that pipes file contents between two commands.  Check
#	that the files were "installed" correctly.
#

# $Id: t0025.t,v 1.11 2001/05/25 14:50:13 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons qw($_rules);
use Config;

$test = Test::Cmd::Cons->new(string => 'Command');

#
$test->write('Construct', <<_EOF_);
DefaultRules(RuleSet("$_rules"));

\%env_hash = ( ${\$test->cons_env} );
\$env_hash{PLSUFFIX} = 'pl';
\$env_hash{FOO} = 'foo';
\$env_hash{TMPFILE} = 'tmpfile';
\$env = new cons ( \%env_hash );
Command \$env 'foo.\%PLSUFFIX', '\%FOO.in', qq(
	\Q$^X\E -MFile::Copy -e "copy('%<', '%TMPFILE') && exit 0"
	\Q$^X\E -MFile::Copy -e "copy('%TMPFILE', '%>') && exit 0"
);
Command \$env 'bar.pl', 'bar.in', qq(
	\Q$^X\E -MFile::Copy -e "copy('%<', \\\*STDOUT) && exit 0" | \Q$^X\E -MFile::Copy -e "copy(\\\*STDIN, '%>') && exit 0"
);
_EOF_

$test->write('foo.in', <<_EOF_);
$Config{startperl}
print "foo.in\\n";
exit (0);
_EOF_

$test->write('bar.in', <<_EOF_);
$Config{startperl}
print "bar.in\\n";
exit (0);
_EOF_

#
$test->run(targets => ".");

$test->execute(prog => 'foo.pl', interpreter => $^X, stdout => <<_EOF_);
foo.in
_EOF_

$test->execute(prog => 'bar.pl', interpreter => $^X, stdout => <<_EOF_);
bar.in
_EOF_

#
$test->pass;
__END__
