#! /usr/bin/env perl
#
#	Use perl to print all combinations of %[<>0-9](:[df]?)?
#	variables into various files.
#

# $Id: t0036.t,v 1.14 2001/05/25 14:50:13 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons qw($_rules);

$test = Test::Cmd::Cons->new(string => '%-variables');

$test->subdir('sub', ['sub', 'dir']);

#
$sub_dir = $test->catfile('sub', 'dir');
$sub_zero = $test->catfile('sub', 'zero');
$sub_zero_out = $test->catfile('sub', 'zero.out');
$sub_dir_right = $test->catfile('sub', 'dir', 'right');
$sub_dir_right_out = $test->catfile('sub', 'dir', 'right.out');

$workdir_right_out = $test->catfile($test->workdir, 'right.out');
$workdir_sub_zero_out = $test->catfile($test->workdir, $sub_zero_out);
$workdir_sub_dir_right_out = $test->catfile($test->workdir, $sub_dir_right_out);

$foo_1 = 'foo.1';
$foo_2 = 'foo.2';
$foo_3 = 'foo.3';
$foo_4 = 'foo.4';
$foo_5 = 'foo.5';
$foo_6 = 'foo.6';
$foo_7 = 'foo.7';
$foo_8 = 'foo.8';
$foo_9 = 'foo.9';
$foo_10 = 'foo.10';
$sub_foo = $test->catfile('sub', 'foo');
$sub_foo_4 = $test->catfile('sub', $foo_4);
$sub_foo_5 = $test->catfile('sub', $foo_5);
$sub_foo_6 = $test->catfile('sub', $foo_6);
$sub_dir_foo = $test->catfile('sub', 'dir', 'foo');
$sub_dir_foo_7 = $test->catfile('sub', 'dir', $foo_7);
$sub_dir_foo_8 = $test->catfile('sub', 'dir', $foo_8);
$sub_dir_foo_9 = $test->catfile('sub', 'dir', $foo_9);

$workdir_foo_1 = $test->catfile($test->workdir, $foo_1);
$workdir_foo_2 = $test->catfile($test->workdir, $foo_2);
$workdir_foo_3 = $test->catfile($test->workdir, $foo_3);
$workdir_sub_foo_4 = $test->catfile($test->workdir, $sub_foo_4);
$workdir_sub_foo_5 = $test->catfile($test->workdir, $sub_foo_5);
$workdir_sub_foo_6 = $test->catfile($test->workdir, $sub_foo_6);
$workdir_sub_dir_foo_7 = $test->catfile($test->workdir, $sub_dir_foo_7);
$workdir_sub_dir_foo_8 = $test->catfile($test->workdir, $sub_dir_foo_8);
$workdir_sub_dir_foo_9 = $test->catfile($test->workdir, $sub_dir_foo_9);
$workdir_foo_10 = $test->catfile($test->workdir, $foo_10);

#
$test->write('Construct', <<_EOF_);
DefaultRules(RuleSet("$_rules"));

\$env = new cons ( ${\$test->cons_env} );
\@deps = qw(
	$foo_1
	$foo_2
	$foo_3
	$sub_foo_4
	$sub_foo_5
	$sub_foo_6
	$sub_dir_foo_7
	$sub_dir_foo_8
	$sub_dir_foo_9
	$foo_10
);
Command \$env 'out_left', \@deps, qq(
	\Q$^X\E -e "print '========== left_bracket', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'left_bracket is %<', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'left_bracket:b is %<:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'left_bracket:d is %<:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'left_bracket:f is %<:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'left_bracket:s is %<:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'left_bracket:F is %<:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'left_bracket:a is %<:a', qq(\\\\n)" >> %>
);
Command \$env 'right.out', \@deps, qq(
	\Q$^X\E -e "print '========== right_bracket', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket is %>', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:b is %>:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:d is %>:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:f is %>:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:s is %>:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:F is %>:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:a is %>:a', qq(\\\\n)" >> %>
);
Command \$env '$sub_zero_out', \@deps, qq(
	\Q$^X\E -e "print '========== 0', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '0 is %0', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '0:b is %0:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '0:d is %0:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '0:f is %0:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '0:s is %0:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '0:F is %0:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '0:a is %0:a', qq(\\\\n)" >> %>
);
Command \$env '$sub_dir_right_out', \@deps, qq(
	\Q$^X\E -e "print '========== right_bracket', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket is %>', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:b is %>:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:d is %>:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:f is %>:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:s is %>:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:F is %>:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print 'right_bracket:a is %>:a', qq(\\\\n)" >> %>
);
Command \$env 'out_numbers', \@deps, qq(
	\Q$^X\E -e "print '========== 1', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '1 is %1', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '1:b is %1:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '1:d is %1:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '1:f is %1:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '1:s is %1:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '1:F is %1:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '1:a is %1:a', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '========== 2', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '2 is %2', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '2:b is %2:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '2:d is %2:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '2:f is %2:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '2:s is %2:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '2:F is %2:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '2:a is %2:a', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '========== 3', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '3 is %3', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '3:b is %3:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '3:d is %3:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '3:f is %3:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '3:s is %3:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '3:F is %3:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '3:a is %3:a', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '========== 4', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '4 is %4', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '4:b is %4:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '4:d is %4:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '4:f is %4:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '4:s is %4:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '4:F is %4:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '4:a is %4:a', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '========== 5', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '5 is %5', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '5:b is %5:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '5:d is %5:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '5:f is %5:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '5:s is %5:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '5:F is %5:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '5:a is %5:a', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '========== 6', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '6 is %6', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '6:b is %6:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '6:d is %6:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '6:f is %6:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '6:s is %6:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '6:F is %6:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '6:a is %6:a', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '========== 7', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '7 is %7', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '7:b is %7:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '7:d is %7:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '7:f is %7:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '7:s is %7:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '7:F is %7:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '7:a is %7:a', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '========== 8', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '8 is %8', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '8:b is %8:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '8:d is %8:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '8:f is %8:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '8:s is %8:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '8:F is %8:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '8:a is %8:a', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '========== 9', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '9 is %9', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '9:b is %9:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '9:d is %9:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '9:f is %9:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '9:s is %9:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '9:F is %9:F', qq(\\\\n)" >> %>
	\Q$^X\E -e "print '9:a is %9:a', qq(\\\\n)" >> %>
);
Command \$env 'out_list', \@deps, qq(
	\Q$^X\E -e "print '========== 1 3 5 7 9 left_bracket', qq(\\\\n)" >> %>
	\Q$^X\E -e "'%1 %3 %5 %7 %9'; print 'after 1 3 5 7 9, left_bracket is %<', qq(\\\\n)" >> %>
	\Q$^X\E -e "'%1 %3 %5 %7 %9'; print 'after 1 3 5 7 9, left_bracket:b is %<:b', qq(\\\\n)" >> %>
	\Q$^X\E -e "'%1 %3 %5 %7 %9'; print 'after 1 3 5 7 9, left_bracket:d is %<:d', qq(\\\\n)" >> %>
	\Q$^X\E -e "'%1 %3 %5 %7 %9'; print 'after 1 3 5 7 9, left_bracket:f is %<:f', qq(\\\\n)" >> %>
	\Q$^X\E -e "'%1 %3 %5 %7 %9'; print 'after 1 3 5 7 9, left_bracket:s is %<:s', qq(\\\\n)" >> %>
	\Q$^X\E -e "'%1 %3 %5 %7 %9'; print 'after 1 3 5 7 9, left_bracket:F is %<:F', qq(\\\\n)" >> %>
);
_EOF_

$test->write($foo_1, <<_EOF_);
This is the $foo_1 file.
_EOF_

$test->write($foo_2, <<_EOF_);
This is the $foo_2 file.
_EOF_

$test->write($foo_3, <<_EOF_);
This is the $foo_3 file.
_EOF_

$test->write($sub_foo_4, <<_EOF_);
This is the $sub_foo_4 file.
_EOF_

$test->write($sub_foo_5, <<_EOF_);
This is the $sub_foo_5 file.
_EOF_

$test->write($sub_foo_6, <<_EOF_);
This is the $sub_foo_6 file.
_EOF_

$test->write($sub_dir_foo_7, <<_EOF_);
This is the $sub_dir_foo_7 file.
_EOF_

$test->write($sub_dir_foo_8, <<_EOF_);
This is the $sub_dir_foo_8 file.
_EOF_

$test->write($sub_dir_foo_9, <<_EOF_);
This is the $sub_dir_foo_9 file.
_EOF_

$test->write($foo_10, <<_EOF_);
This is the $foo_10 file.
_EOF_

#
$test->run(targets => ".");

$test->file_matches('out_left', <<_EOF_);
========== left_bracket
left_bracket is $foo_1 $foo_2 $foo_3 $sub_foo_4 $sub_foo_5 $sub_foo_6 $sub_dir_foo_7 $sub_dir_foo_8 $sub_dir_foo_9 $foo_10
left_bracket:b is foo foo foo $sub_foo $sub_foo $sub_foo $sub_dir_foo $sub_dir_foo $sub_dir_foo foo
left_bracket:d is . . . sub sub sub $sub_dir $sub_dir $sub_dir .
left_bracket:f is $foo_1 $foo_2 $foo_3 $foo_4 $foo_5 $foo_6 $foo_7 $foo_8 $foo_9 $foo_10
left_bracket:s is .1 .2 .3 .4 .5 .6 .7 .8 .9 .10
left_bracket:F is foo foo foo foo foo foo foo foo foo foo
left_bracket:a is $workdir_foo_1 $workdir_foo_2 $workdir_foo_3 $workdir_sub_foo_4 $workdir_sub_foo_5 $workdir_sub_foo_6 $workdir_sub_dir_foo_7 $workdir_sub_dir_foo_8 $workdir_sub_dir_foo_9 $workdir_foo_10
_EOF_

$test->file_matches('right.out', <<_EOF_);
========== right_bracket
right_bracket is right.out
right_bracket:b is right
right_bracket:d is .
right_bracket:f is right.out
right_bracket:s is .out
right_bracket:F is right
right_bracket:a is $workdir_right_out
_EOF_

$test->file_matches($sub_zero_out, <<_EOF_);
========== 0
0 is $sub_zero_out
0:b is $sub_zero
0:d is sub
0:f is zero.out
0:s is .out
0:F is zero
0:a is $workdir_sub_zero_out
_EOF_

$test->file_matches($sub_dir_right_out, <<_EOF_);
========== right_bracket
right_bracket is $sub_dir_right_out
right_bracket:b is $sub_dir_right
right_bracket:d is $sub_dir
right_bracket:f is right.out
right_bracket:s is .out
right_bracket:F is right
right_bracket:a is $workdir_sub_dir_right_out
_EOF_

$test->file_matches('out_numbers', <<_EOF_);
========== 1
1 is $foo_1
1:b is foo
1:d is .
1:f is $foo_1
1:s is .1
1:F is foo
1:a is $workdir_foo_1
========== 2
2 is $foo_2
2:b is foo
2:d is .
2:f is $foo_2
2:s is .2
2:F is foo
2:a is $workdir_foo_2
========== 3
3 is $foo_3
3:b is foo
3:d is .
3:f is $foo_3
3:s is .3
3:F is foo
3:a is $workdir_foo_3
========== 4
4 is $sub_foo_4
4:b is $sub_foo
4:d is sub
4:f is $foo_4
4:s is .4
4:F is foo
4:a is $workdir_sub_foo_4
========== 5
5 is $sub_foo_5
5:b is $sub_foo
5:d is sub
5:f is $foo_5
5:s is .5
5:F is foo
5:a is $workdir_sub_foo_5
========== 6
6 is $sub_foo_6
6:b is $sub_foo
6:d is sub
6:f is $foo_6
6:s is .6
6:F is foo
6:a is $workdir_sub_foo_6
========== 7
7 is $sub_dir_foo_7
7:b is $sub_dir_foo
7:d is $sub_dir
7:f is $foo_7
7:s is .7
7:F is foo
7:a is $workdir_sub_dir_foo_7
========== 8
8 is $sub_dir_foo_8
8:b is $sub_dir_foo
8:d is $sub_dir
8:f is $foo_8
8:s is .8
8:F is foo
8:a is $workdir_sub_dir_foo_8
========== 9
9 is $sub_dir_foo_9
9:b is $sub_dir_foo
9:d is $sub_dir
9:f is $foo_9
9:s is .9
9:F is foo
9:a is $workdir_sub_dir_foo_9
_EOF_

$test->file_matches('out_list', <<_EOF_);
========== 1 3 5 7 9 left_bracket
after 1 3 5 7 9, left_bracket is $foo_2 $sub_foo_4 $sub_foo_6 $sub_dir_foo_8 $foo_10
after 1 3 5 7 9, left_bracket:b is foo $sub_foo $sub_foo $sub_dir_foo foo
after 1 3 5 7 9, left_bracket:d is . sub sub $sub_dir .
after 1 3 5 7 9, left_bracket:f is $foo_2 $foo_4 $foo_6 $foo_8 $foo_10
after 1 3 5 7 9, left_bracket:s is .2 .4 .6 .8 .10
after 1 3 5 7 9, left_bracket:F is foo foo foo foo foo
_EOF_

#
$test->pass;
__END__
