#! /usr/bin/env perl
#
#	Create four target scripts, two in Construct and two in Conscript,
#	each by copying a source file to the destination.  Specify an
#	AfterBuild action of printing a string for each, one a direct
#	print, and the other three by calling a same-named subroutine
#	defined once each in Construct and Conscript.  The Conscript file
#	specifies both scripts on the same AfterBuild line.  Run Cons;
#	check that standard output contains the "build" command and
#	the AfterBuild outputs, including separate output from the two
#	print subroutines.  Run the target scripts to make sure they
#	"built" correctly.  Re-run Cons; check that standard output
#	contains the AfterBuild outputs and the Cons up-to-date message.
#

# $Id: t0084.t,v 1.9 2001/05/25 14:50:14 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons qw($_rules);
use Config;

$test = Test::Cmd::Cons->new(string => 'AfterBuild');

$test->subdir('src');

#
$test->write('Construct', <<_EOF_);
DefaultRules(RuleSet("$_rules"));

\$env = new cons ( ${\$test->cons_env} );
Command \$env 'foo.pl', 'foo.in', qq(
	\Q$^X\E -e "use File::Copy; copy('%<', '%>'); exit 0"
);
Command \$env 'goo.pl', 'goo.in', qq(
	\Q$^X\E -e "use File::Copy; copy('%<', '%>'); exit 0"
);
sub printsub {
	print "Construct printsub:  ", \@_, "\\n";
}
AfterBuild \$env 'foo.pl', qq(print "foo.pl AfterBuild\\n";);
AfterBuild \$env 'goo.pl', "printsub('goo.pl AfterBuild');";
Export qw( env );
Build 'Conscript';
_EOF_

$test->write('Conscript', <<_EOF_);
Import qw( env );
Command \$env 'bar.pl', 'bar.in', qq(
	\Q$^X\E -e "use File::Copy; copy('%<', '%>'); exit 0"
);
Command \$env 'car.pl', 'car.in', qq(
	\Q$^X\E -e "use File::Copy; copy('%<', '%>'); exit 0"
);
sub printsub {
	print "Conscript printsub:  ", \@_, "\\n";
}
AfterBuild \$env 'bar.pl', 'car.pl', "printsub('bar.pl car.pl AfterBuild');";
Depends \$env 'bar.pl', 'foo.pl';	# so foo.pl is built first
Depends \$env 'goo.pl', 'bar.pl';	# so bar.pl is built second
Depends \$env 'car.pl', 'goo.pl';	# so goo.pl is built third
_EOF_

$test->write('foo.in', <<_EOF_);
$Config{startperl}
print "foo.in\\n";
exit (0);
_EOF_

$test->write('goo.in', <<_EOF_);
$Config{startperl}
print "goo.in\\n";
exit (0);
_EOF_

$test->write('bar.in', <<_EOF_);
$Config{startperl}
print "bar.in\\n";
exit (0);
_EOF_

$test->write('car.in', <<_EOF_);
$Config{startperl}
print "car.in\\n";
exit (0);
_EOF_

#
$test->run(targets => ".", stdout => <<_EOF_);
$^X -e "use File::Copy; copy('foo.in', 'foo.pl'); exit 0"
foo.pl AfterBuild
$^X -e "use File::Copy; copy('bar.in', 'bar.pl'); exit 0"
Conscript printsub:  bar.pl car.pl AfterBuild
$^X -e "use File::Copy; copy('goo.in', 'goo.pl'); exit 0"
Construct printsub:  goo.pl AfterBuild
$^X -e "use File::Copy; copy('car.in', 'car.pl'); exit 0"
Conscript printsub:  bar.pl car.pl AfterBuild
_EOF_

$test->execute(interpreter => $^X, prog => 'foo.pl', stdout => <<_EOF_);
foo.in
_EOF_

$test->execute(interpreter => $^X, prog => 'goo.pl', stdout => <<_EOF_);
goo.in
_EOF_

$test->execute(interpreter => $^X, prog => 'bar.pl', stdout => <<_EOF_);
bar.in
_EOF_

$test->execute(interpreter => $^X, prog => 'car.pl', stdout => <<_EOF_);
car.in
_EOF_

#
$test->run(targets => ".", stdout => <<_EOF_);
foo.pl AfterBuild
Conscript printsub:  bar.pl car.pl AfterBuild
Construct printsub:  goo.pl AfterBuild
Conscript printsub:  bar.pl car.pl AfterBuild
${\$test->basename}: "." is up-to-date.
_EOF_

#
$test->pass;
__END__
