#! /usr/bin/env perl
#
#	Create a Construct file that uses Command to build three files
#	from Perl code references (one from an anonymous subroutine,
#	and two from a named subroutine) and which also uses a code
#	reference for the ARCOM value in the construction environment.
#	Create a subsidiary Conscript file that builds another file
#	using a different subroutine with the same name as the Construct
#	file's subroutine.  Run Cons and check that all the files were
#	built correctly.  Update the "Library" input file and re-run
#	Cons to make sure the code reference gets executed again.
#
#	NOTE:  THIS IS A TEST OF AN EXPERIMENTAL INTERFACE, STILL SUBJECT
#	TO CHANGE.
#

# $Id: t0085.t,v 1.10 2001/05/25 14:50:14 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons qw($_rules $_a $_o);

$test = Test::Cmd::Cons->new(string => 'perl code ref build actions [EXPERIMENTAL]');

#
$test->write("Construct", <<_EOF_);
DefaultRules(RuleSet("$_rules"));

sub ar_sub {
	my(\$env, \$target, \@sources) = \@_;
	print qq(ar_sub: building \$target\\n);
	open(F, ">\$target");
	print F qq(Construct ar_sub\\n);
	foreach \$src (\@sources) {
		open(IN, "<\$src");
		print F <IN>;
		close(IN);
	}
	close(F);
	return 1;
}
\%env_hash = ( ${\$test->cons_env} );
\$env_hash{'ARCOM'} = \\\&ar_sub;
\$env = new cons ( \%env_hash );
Command \$env 'out1', sub {
	my(\$env, \$target, \@sources) = \@_;
	print qq(sub1: building \$target\\n);
	open(F, ">\$target");
	print F qq(Construct sub1\\n);
	close(F);
	return 1;
};
sub sub2 {
	my(\$env, \$target) = \@_;
	open(F, ">\$target");
	print qq(sub2: building \$target\\n);
	print F qq(Construct sub2\\n);
	close(F);
	return 1;
}
Command \$env 'out2', \\\&sub2;
Command \$env 'out3', \\\&sub2;
Library \$env 'outlib', 'inlib$_o';
Export qw( env );
Build 'Conscript';
_EOF_

$test->write("Conscript", <<_EOF_);
Import qw( env );
sub sub2 {
	my(\$env, \$target, \@sources) = \@_;
	print qq(sub2: building \$target\\n);
	open(F, ">\$target");
	print F qq(Conscript sub2\\n);
	close(F);
	return 1;
}
Command \$env 'out4', \\\&sub2;
_EOF_

$test->write("inlib$_o", <<_EOF_);
inlib 1
_EOF_

#
$test->run(targets => ".", stdout => <<_EOF_, stderr => <<_EOF_);
sub1: building out1
sub2: building out2
sub2: building out3
sub2: building out4
ar_sub: building outlib$_a
_EOF_
_EOF_

$test->file_matches('out1', <<_EOF_);
Construct sub1
_EOF_

$test->file_matches('out2', <<_EOF_);
Construct sub2
_EOF_

$test->file_matches('out3', <<_EOF_);
Construct sub2
_EOF_

$test->file_matches('out4', <<_EOF_);
Conscript sub2
_EOF_

$test->file_matches("outlib$_a", <<_EOF_);
Construct ar_sub
inlib 1
_EOF_

#
$test->up_to_date(targets => ".");

#
$test->write("inlib$_o", <<_EOF_);
inlib 2
_EOF_

$test->run(targets => ".", stdout => <<_EOF_, stderr => <<_EOF_);
ar_sub: building outlib$_a
_EOF_
_EOF_

$test->file_matches("outlib$_a", <<_EOF_);
Construct ar_sub
inlib 2
_EOF_

#
$test->up_to_date(targets => ".");

# PASS THE TEST HERE until we solve the problem of generating signatures
# for code references to detect when a file needs to be rebuilt due to
# a code reference change.  See the comments in cons itself for details.
$test->pass;

#
$test->write("Construct", <<_EOF_);
DefaultRules(RuleSet("$_rules"));

\$env = new cons ( ${\$test->cons_env} );
sub sub1 {
	my \$file = shift;
	open(F, ">\$file");
	print F qq(Construct sub1 again\\n);
	close(F);
	return 1;
}
\$variable = '12345';
Command \$env 'out1',
    qq([perl] open(F, ">%>"); print F qq(\$variable \\\$variable\\n); close(F); 1);
Command \$env 'out2', qq([perl] &sub1('%>'));
#Command \$env 'out3', sub { &printsub('%s', 'out3') };
Export qw( env );
Build 'Conscript';
_EOF_

#
$test->run(targets => ".");

$test->file_matches('out1', <<_EOF_);
12345 12345
_EOF_

$test->file_matches('out2', <<_EOF_);
Construct sub1 <>
_EOF_

#$test->file_matches('out3', <<_EOF_);
#Construct sub1 <out3>
#_EOF_

#
$test->pass;
__END__
