#! /usr/bin/env perl
#
#	Create an empty subdirectory (with a do-nothing Conscript) and a
#	source subdirectory with a source file.  Create Conscript files
#	that Link the source subdirectory and copy the source file to a
#	"build" file using a generated script.	Execute "Cons -t -q" and
#	"Cons -t -q -q" from the empty subdirectory and check that the
#	second invocation suppresses the "nothing to be built" message.
#	From the top level, execute Cons, Cons, and "Cons -r", then
#	once each again for the -q option, and again for -qq.  Check
#	that appropriate messages are suppressed each time.  Execute
#	"Cons" in various combinations with the -t option from the
#	source subdirectory; check that the "Linked targets" message is
#	suppressed appropriately.
#
#	NOTE:  THIS TEST EXAMINES THE ACTIONS USED TO BUILD FILES.
#

# $Id: t0086.t,v 1.11 2001/05/25 14:50:14 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons qw($_rules $_exe $_o);

$test = Test::Cmd::Cons->new(string => '-q');

$test->subdir('nothing', 'src');

$build_Conscript = $test->catfile('build', 'Conscript');
$build_aaa = $test->catfile('build', 'aaa');
$build_aaa_in = $test->catfile('build', 'aaa.in');
$install_aaa = $test->catfile('install', 'aaa');
$src__consign = $test->catfile('src', '.consign');

$workdir_mycopy_pl = $test->workpath('mycopy.pl');

#
$test->write('mycopy.pl', <<_EOF_);
use File::Copy;
copy(\$ARGV[0], \$ARGV[1]);
exit 0;
_EOF_

#
$test->write('Construct', <<_EOF_);
DefaultRules(RuleSet("$_rules"));

\$env = new cons ( ${\$test->cons_env} );
Export qw( env );
Link 'build' => 'src';
Build '$build_Conscript';
_EOF_

$test->write(['nothing', 'Conscript'], <<_EOF_);
Import qw( env );
_EOF_

$test->write(['src', 'Conscript'], <<_EOF_);
Import qw( env );
Command \$env 'aaa', 'aaa.in', qq(
	\Q$^X\E \Q$workdir_mycopy_pl\E %< %>
);
Install \$env '#install', 'aaa';
_EOF_

$test->write(['src', 'aaa.in'], <<'_EOF_');
src/aaa.in
_EOF_



#
$test->run('chdir' => "nothing", flags => "-t -q", targets => ".", stdout => <<_EOF_);
${\$test->basename}: Entering directory `${\$test->workdir}'
${\$test->basename}: nothing to be built in "nothing".
_EOF_

$test->must_not_exist($install_aaa);
$test->must_not_exist($build_aaa);

$test->must_not_exist($src__consign);

#
$test->run('chdir' => "nothing", flags => "-t -q -q", targets => ".", stdout => <<_EOF_);
${\$test->basename}: Entering directory `${\$test->workdir}'
_EOF_

$test->must_not_exist($install_aaa);
$test->must_not_exist($build_aaa);

$test->must_not_exist($src__consign);



#
$test->run(targets => ".", stdout => <<_EOF_);
$^X $workdir_mycopy_pl $build_aaa_in $build_aaa
Install $build_aaa as $install_aaa
_EOF_

$test->file_matches($install_aaa, "src/aaa.in\n");

$test->must_not_exist($src__consign);

$test->run(targets => ".", stdout => <<_EOF_);
${\$test->basename}: "." is up-to-date.
_EOF_

$test->must_not_exist($src__consign);

$test->run(flags => "-r", targets => ".", stdout => <<_EOF_);
Removed $build_aaa
Removed $install_aaa
_EOF_

$test->must_not_exist($install_aaa);
$test->must_not_exist($build_aaa);

$test->must_not_exist($src__consign);

#
$test->run(flags => "-q", targets => ".", stdout => <<_EOF_);
$^X $workdir_mycopy_pl $build_aaa_in $build_aaa
_EOF_

$test->file_matches($install_aaa, "src/aaa.in\n");

$test->must_not_exist($src__consign);

$test->run(flags => "-q", targets => ".", stdout => <<_EOF_);
${\$test->basename}: "." is up-to-date.
_EOF_

$test->must_not_exist($src__consign);

$test->run(flags => "-q -r", targets => ".", stdout => <<_EOF_);
_EOF_

$test->must_not_exist($install_aaa);
$test->must_not_exist($build_aaa);

$test->must_not_exist($src__consign);

#
$test->run(flags => "-qq", targets => ".", stdout => <<_EOF_);
_EOF_

$test->file_matches($install_aaa, "src/aaa.in\n");

$test->must_not_exist($src__consign);

$test->run(flags => "-qq", targets => ".", stdout => <<_EOF_);
_EOF_

$test->must_not_exist($src__consign);

$test->run(flags => "-qqr", targets => ".", stdout => <<_EOF_);
_EOF_

$test->must_not_exist($install_aaa);
$test->must_not_exist($build_aaa);

$test->must_not_exist($src__consign);



#
$test->run('chdir' => "src", flags => "-t", targets => ".", stdout => <<_EOF_);
${\$test->basename}: Entering directory `${\$test->workdir}'
Linked targets: build
$^X $workdir_mycopy_pl $build_aaa_in $build_aaa
_EOF_

$test->file_matches($build_aaa, "src/aaa.in\n");

$test->must_not_exist($src__consign);

$test->run('chdir' => "src", flags => "-t", targets => ".", stdout => <<_EOF_);
${\$test->basename}: Entering directory `${\$test->workdir}'
Linked targets: build
${\$test->basename}: "build" is up-to-date.
_EOF_

$test->must_not_exist($src__consign);

$test->run('chdir' => "src", flags => "-t -r", targets => ".", stdout => <<_EOF_);
${\$test->basename}: Entering directory `${\$test->workdir}'
Linked targets: build
Removed $build_aaa
_EOF_

$test->must_not_exist($install_aaa);
$test->must_not_exist($build_aaa);

$test->must_not_exist($src__consign);

#
$test->run('chdir' => "src", flags => "-t -q", targets => ".", stdout => <<_EOF_);
${\$test->basename}: Entering directory `${\$test->workdir}'
$^X $workdir_mycopy_pl $build_aaa_in $build_aaa
_EOF_

$test->file_matches($build_aaa, "src/aaa.in\n");

$test->must_not_exist($src__consign);

$test->run('chdir' => "src", flags => "-tq", targets => ".", stdout => <<_EOF_);
${\$test->basename}: Entering directory `${\$test->workdir}'
${\$test->basename}: "build" is up-to-date.
_EOF_

$test->must_not_exist($src__consign);

$test->run('chdir' => "src", flags => "-t -q -r", targets => ".", stdout => <<_EOF_);
${\$test->basename}: Entering directory `${\$test->workdir}'
_EOF_

$test->must_not_exist($install_aaa);
$test->must_not_exist($build_aaa);

$test->must_not_exist($src__consign);



#
# XXX - For completeness, there should be a test for
# suppressing the "Retrieved ... from cache" message.
# It's not really crucial right now, though.
#



#
$test->pass;
__END__
