#! /usr/bin/env perl
#
#	Sub-class the "cons" class and override its internal _Object
#	method with one that calls that creates a marker file and then
#	calls the super-class method to do the real work.  Create a
#	Program from two Objects files.  Run cons.  Check that the
#	Program was build correctly and that the marker file was created,
#	indicating our overridden method was called.
#

# $Id: t0087.t,v 1.5 2001/05/25 14:50:14 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons qw($_rules);

$test = Test::Cmd::Cons->new(string => 'override sub-classed methods');

#
$test->write('Construct', <<_EOF_);
DefaultRules(RuleSet("$_rules"));

package my_cons;
\@ISA = qw( cons );

sub _Object {
    my(\$env) = shift;
    open(F, ">my_cons"); print F "_Object\n"; close(F);
    \$env->SUPER::_Object(\@_);
}

package script;
\$env = new my_cons ( ${\$test->cons_env} );
Export qw( env );
\@ofiles = Objects \$env qw(foo.c aaa.c);
Program \$env 'foo', \@ofiles;
_EOF_

$test->write('foo.c', <<'_EOF_');
extern void aaa(void);
extern void bbb(void);
int
main(int argc, char *argv[])
{
	aaa();
	printf("foo.c\n");
	exit (0);
}
_EOF_

$test->write('aaa.c', <<'_EOF_');
void
aaa(void)
{
	printf("aaa.c\n");
}
_EOF_

#
$test->run(targets => ".");

$test->execute(prog => 'foo', stdout => <<_EOF_);
aaa.c
foo.c
_EOF_

# If this file can't be read, our sub-classed method was not called.
$test->file_matches('my_cons', <<_EOF_);
_Object
_EOF_

$test->pass;
__END__
