#! /usr/bin/env perl
#
#	Use a Command method that copies an input file to "build" an
#	output file.  Run Cons to generate the output file.  Update the
#	input file.  Change the source file's timestamp in the .consign
#	file to be the actual modification time of the file (simulating
#	an update to the file within the same second of its initial
#	building).  Re-run Cons and make sure that, by default, it
#	re-builds the output file (i.e., the matching timestamp in the
#	.consign file doesn't fool it into thinking that the modified
#	source file is actually up-to-date).
#
#	WARNING:  THIS TEST MODIFIES A .consign FILE DIRECTLY.
#

# $Id: t0088.t,v 1.8 2001/05/25 14:50:14 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons qw($_rules);
use Config;

$test = Test::Cmd::Cons->new(string => 'same-second update');

#
$test->write('Construct', <<_EOF_);
DefaultRules(RuleSet("$_rules"));

\$env = new cons ( ${\$test->cons_env} );
Command \$env 'foo.out', 'foo.in', qq(
	\Q$^X\E -MFile::Copy -e "copy('%<', '%>') && exit 0"
);
_EOF_

$test->write('foo.in', <<_EOF_);
foo.in 1
_EOF_

#
$test->run(targets => ".");

$test->file_matches('foo.out', <<_EOF_);
foo.in 1
_EOF_

#
$test->write('foo.in', <<_EOF_);
foo.in 2
_EOF_

$test->adjust_consign_times('.', 'foo.in');

#
$test->run(targets => ".");

$test->file_matches('foo.out', <<_EOF_);
foo.in 2
_EOF_

#
$test->pass;
__END__
