#! /usr/bin/env perl
#
#	Create a subsidiary Perl file-copy script.  Create a Construct
#	file that uses it to copy an input that copies an input file to
#	an output file.  Run Cons to create the output file; examine the
#	output to see that it just contains the actions.  Update the
#	input file and run "cons -S md5::debug" to rebuild the output;
#	examine the output to see that it also contains signature
#	debugging information.  Update the input file again, set the
#	environment variable CONS_SIG_DEBUG to a file name, and re-run
#	"cons -S md5::debug" to rebuild the output file; examine the
#	output to see that it only contains the action, and examine the
#	specified CONS_SIG_DEBUG file name to see that it contains just
#	the signature debug info.
#
#	NOTE:  THIS TEST EXAMINES THE ACTIONS USED TO BUILD FILES.
#

# $Id: t0089.t,v 1.9 2001/05/25 14:50:14 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons qw($_rules $_exe);
use Config;

$test = Test::Cmd::Cons->new(string => '-S md5::debug',
			     match_sub => \&Test::Cmd::diff_regex);

$copy_pl = $test->workpath('copy.pl');

#
$test->write('copy.pl', <<_EOF_);
$Config{startperl}
use File::Copy;
my \$in = shift;
my \$out = shift;
copy(\$in, \$out) || die "cannot copy \$in to \$out: \$!";
exit 0;
_EOF_

$test->write('Construct', <<_EOF_);
DefaultRules(RuleSet("$_rules"));

\$env = new cons ( ${\$test->cons_env} );
Command \$env 'foo.out', 'foo.in', "\Q$^X\E \Q$copy_pl\E %< %>";
Ignore qw([Pp]erl$_exe);
_EOF_

$test->write('foo.in', <<_EOF_);
foo.in 1
_EOF_

#
$test->run(targets => ".", stdout => <<_EOF_);
\Q$^X\E \Q$copy_pl\E foo.in foo.out
_EOF_

$test->file_matches('foo.out', <<_EOF_);
foo.in 1
_EOF_

#
$test->write('foo.in', <<_EOF_);
foo.in 2
_EOF_

#
$test->run(flags => '-S md5::debug', targets => ".", stdout => <<_EOF_);
sig::md5::srcsig\\\(foo.in\\\)
\\\t=> \\\|[0-9a-f]{32}\\\|
sig::md5::collect\\\([0-9a-f]{32},
                  (\Q$^X\E[0-9]+)?,
                  \Q$^X\E \Q$copy_pl\E %< %>\\\)
\\\t=> \\\|[0-9a-f]{32}\\\|
sig::md5::current\\\(\\\|[0-9a-f]{32}\\\|
               eq \\\|[0-9a-f]{32}\\\|\\\)
\Q$^X\E \Q$copy_pl\E foo.in foo.out
_EOF_

$test->file_matches('foo.out', <<_EOF_);
foo.in 2
_EOF_

#
$test->write('foo.in', <<_EOF_);
foo.in 3
_EOF_

$ENV{CONS_SIG_DEBUG} = "sig_debug";

#
$test->run(flags => '-S md5::debug', targets => ".", stdout => <<_EOF_);
\Q$^X\E \Q$copy_pl\E foo.in foo.out
_EOF_

$test->file_matches('sig_debug', <<_EOF_);
sig::md5::srcsig\\\(foo.in\\\)
\\\t=> \\\|[0-9a-f]{32}\\\|
sig::md5::collect\\\([0-9a-f]{32},
                  (\Q$^X\E[0-9]+)?,
                  \Q$^X\E \Q$copy_pl\E %< %>\\\)
\\\t=> \\\|[0-9a-f]{32}\\\|
sig::md5::current\\\(\\\|[0-9a-f]{32}\\\|
               eq \\\|[0-9a-f]{32}\\\|\\\)
_EOF_

$test->file_matches('foo.out', <<_EOF_);
foo.in 3
_EOF_

#
$test->pass;
__END__
