#! /usr/bin/env perl
#
#	In three identical subdirectories, build two files each from
#	input files and intermediate derived files by simply stripping
#	off comments in each "build" step.  Specify one subdirectory's
#	Conscript environment with a SIGNATURE of 'build', another with
#	'content', and the third with 'stored-content.'  Build all
#	the files.  Update the first stripped comment in one of the
#	input files (in eachd subdirectory).  Rebuild, and verify that
#	the 'build' directory rebuilt the output file, but that the
#	'content' and 'stored-content' files stopped after re-generating
#	the same intermediate file.  Update both intermediate files
#	(in each subdirectory) with a new second stripped comment.
#	Adjust the .consign time on one of the intermediate files to
#	match the modification time.  Rebuild, and verify that the
#	'build' directory did not rebuild anything (source file did not
#	change), and that the 'content' directory rebuilt both output
#	files, but that the 'stored-content' directory only rebuilt
#	the output file whose intermediate timestamp had changed.
#	Update an intermediate file with a new second stripped comment.
#	Rebuild, and verify that both the 'content' and 'stored-content'
#	directories rebuilt the output file.
#
#	WARNING:  THIS TEST MODIFIES .consign FILES DIRECTLY.
#

# $Id: t0090.t,v 1.4 2001/05/25 14:50:14 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons;
use Config;

$test = Test::Cmd::Cons->new(string => 'SIGNATURE',
			     match_sub => \&Test::Cmd::diff_regex);

@dirs = (
	'build',
	'content',
	'stored-content',
);

$test->subdir(@dirs);

#
$build_Conscript = $test->catfile('build', 'Conscript');
$content_Conscript = $test->catfile('content', 'Conscript');
$stored_content_Conscript = $test->catfile('stored-content', 'Conscript');

$build_in1 = $test->catfile('build', 'in1');
$build_in2 = $test->catfile('build', 'in2');
$build_mid1 = $test->catfile('build', 'mid1');
$build_mid2 = $test->catfile('build', 'mid2');
$build_out1 = $test->catfile('build', 'out1');
$build_out2 = $test->catfile('build', 'out2');
$build_output = $test->catfile('build', 'output');

$stored_content_in1 = $test->catfile('stored-content', 'in1');
$stored_content_in2 = $test->catfile('stored-content', 'in2');
$stored_content_mid1 = $test->catfile('stored-content', 'mid1');
$stored_content_mid2 = $test->catfile('stored-content', 'mid2');
$stored_content_out1 = $test->catfile('stored-content', 'out1');
$stored_content_out2 = $test->catfile('stored-content', 'out2');
$stored_content_output = $test->catfile('stored-content', 'output');

$content_in1 = $test->catfile('content', 'in1');
$content_in2 = $test->catfile('content', 'in2');
$content_mid1 = $test->catfile('content', 'mid1');
$content_mid2 = $test->catfile('content', 'mid2');
$content_out1 = $test->catfile('content', 'out1');
$content_out2 = $test->catfile('content', 'out2');
$content_output = $test->catfile('content', 'output');

#
$test->write('b1.pl', <<_EOF_);
$Config{startperl}
while (<>) {
	s/#b1.*//;
	print;
}
exit(0);
_EOF_

$test->write('b2.pl', <<_EOF_);
$Config{startperl}
while (<>) {
	s/#b2.*//;
	print;
}
exit(0);
_EOF_

$test->write('Construct', <<_EOF_);
Build qw(
	$build_Conscript
	$content_Conscript
	$stored_content_Conscript
);
_EOF_

foreach $dir (@dirs) {
	$test->write([$dir, 'Conscript'], <<_EOF_);
\%env_hash = ( ${\$test->cons_env} );
\$env_hash{SIGNATURE} = [ '*' => '$dir' ];
\$env = new cons ( \%env_hash );
Command \$env 'mid1', 'in1', qq( \Q$^X\E b1.pl %< > %> );
Command \$env 'mid2', 'in2', qq( \Q$^X\E b1.pl %< > %> );
Command \$env 'out1', 'mid1', qq( \Q$^X\E b2.pl %< > %> );
Command \$env 'out2', 'mid2', qq( \Q$^X\E b2.pl %< > %> );
Command \$env 'output', 'out1', 'out2', qq( \Q$^X\E b1.pl %< > %> );
_EOF_

	$test->write([$dir, 'in1'], <<_EOF_);
input 1#b2 comment #b1 comment
_EOF_

	$test->write([$dir, 'in2'], <<_EOF_);
input 2#b2 comment #b1 comment
_EOF_
}

$test->run(targets => ".", stdout => <<_EOF_);
\Q$^X b1.pl $build_in1 > $build_mid1\E
\Q$^X b1.pl $build_in2 > $build_mid2\E
\Q$^X b2.pl $build_mid1 > $build_out1\E
\Q$^X b2.pl $build_mid2 > $build_out2\E
\Q$^X b1.pl $build_out1 $build_out2 > $build_output\E
\Q$^X b1.pl $content_in1 > $content_mid1\E
\Q$^X b1.pl $content_in2 > $content_mid2\E
\Q$^X b2.pl $content_mid1 > $content_out1\E
\Q$^X b2.pl $content_mid2 > $content_out2\E
\Q$^X b1.pl $content_out1 $content_out2 > $content_output\E
\Q$^X b1.pl $stored_content_in1 > $stored_content_mid1\E
\Q$^X b1.pl $stored_content_in2 > $stored_content_mid2\E
\Q$^X b2.pl $stored_content_mid1 > $stored_content_out1\E
\Q$^X b2.pl $stored_content_mid2 > $stored_content_out2\E
\Q$^X b1.pl $stored_content_out1 $stored_content_out2 > $stored_content_output\E
_EOF_

foreach $dir (@dirs) {
	$test->file_matches([$dir, 'out1'], <<_EOF_);
input 1
_EOF_

	$test->file_matches([$dir, 'out2'], <<_EOF_);
input 2
_EOF_

	$test->file_matches([$dir, 'output'], <<_EOF_);
input 1
input 2
_EOF_
}

$test->up_to_date(targets => ".");

#
$test->sleep;

foreach $dir (@dirs) {
	$test->write([$dir, 'in1'], <<_EOF_);
input 1#b2 comment #b1 new comment
_EOF_
}

#
$test->run(targets => ".", stdout => <<_EOF_);
\Q$^X b1.pl $build_in1 > $build_mid1\E
\Q$^X b2.pl $build_mid1 > $build_out1\E
\Q$^X b1.pl $build_out1 $build_out2 > $build_output\E
\Q$^X b1.pl $content_in1 > $content_mid1\E
\Q$^X b1.pl $stored_content_in1 > $stored_content_mid1\E
_EOF_

#
$test->sleep;

foreach $dir (@dirs) {
	$test->write([$dir, 'mid1'], <<_EOF_);
input 1#b2 new comment
_EOF_
	$test->write([$dir, 'mid2'], <<_EOF_);
input 2#b2 new comment
_EOF_
	$test->adjust_consign_times($dir, 'mid1');
}

#
$test->run(targets => ".", stdout => <<_EOF_);
\Q$^X b2.pl $content_mid1 > $content_out1\E
\Q$^X b2.pl $content_mid2 > $content_out2\E
\Q$^X b2.pl $stored_content_mid2 > $stored_content_out2\E
_EOF_

#
$test->sleep;

foreach $dir (@dirs) {
	$test->write([$dir, 'mid1'], <<_EOF_);
input 1#b2 newer comment
_EOF_
}

#
$test->run(targets => ".", stdout => <<_EOF_);
\Q$^X b2.pl $content_mid1 > $content_out1\E
\Q$^X b2.pl $stored_content_mid1 > $stored_content_out1\E
_EOF_

#
$test->pass;
__END__
