#! /usr/bin/env perl
#
#	Create a Construct and Conscript that build three output files
#	each in the current directory and a subdirectory by copying
#	separate input files, each with a different file suffix.  The
#	Construct file's SourceSignature specifies that source file
#	suffix .xxx and .yyy should use the .consign file, but .yyy in
#	the subdir and .zzz should not.  Build the output files and
#	verify their contents.  Change the contents of all the input
#	files.  Update the .consign files to match the new modification
#	times of the input files (simulating an erroneous match).
#	Re-run Cons, and check that only the source files that use
#	their content to generate the signature had their output files
#	updated.
#
#	WARNING:  THIS TEST MODIFIES .consign FILES DIRECTLY.
#

# $Id: t0091.t,v 1.4 2001/05/25 14:50:14 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons;
use Config;

$test = Test::Cmd::Cons->new(string => 'SourceSignature');

$test->subdir('subdir');

$subdir_Conscript = $test->catfile('subdir', 'Conscript');

#
$test->write('Construct', <<_EOF_);
SourceSignature '*.xxx' => 'stored-content',
		'subdir/*.yyy', => 'content',
		'*.yyy' => 'stored-content',
		'*.zzz' => 'content',
		'*' => 'stored-content';
\$env = new cons ( ${\$test->cons_env} );
Command \$env 'f1.out', 'f1.xxx', qq(
	\Q$^X\E -MFile::Copy -e "copy('%<', '%>') && exit 0"
);
Command \$env 'f2.out', 'f2.yyy', qq(
	\Q$^X\E -MFile::Copy -e "copy('%<', '%>') && exit 0"
);
Command \$env 'f3.out', 'f3.zzz', qq(
	\Q$^X\E -MFile::Copy -e "copy('%<', '%>') && exit 0"
);
Export qw( env );
Build '$subdir_Conscript';
_EOF_

$test->write(['subdir', 'Conscript'], <<_EOF_);
Import qw( env );
Command \$env 'b1.out', 'b1.xxx', qq(
	\Q$^X\E -MFile::Copy -e "copy('%<', '%>') && exit 0"
);
Command \$env 'b2.out', 'b2.yyy', qq(
	\Q$^X\E -MFile::Copy -e "copy('%<', '%>') && exit 0"
);
Command \$env 'b3.out', 'b3.zzz', qq(
	\Q$^X\E -MFile::Copy -e "copy('%<', '%>') && exit 0"
);
_EOF_

$test->write('f1.xxx', <<_EOF_);
f1.xxx 1
_EOF_

$test->write('f2.yyy', <<_EOF_);
f2.yyy 1
_EOF_

$test->write('f3.zzz', <<_EOF_);
f3.zzz 1
_EOF_

$test->write(['subdir', 'b1.xxx'], <<_EOF_);
subdir/b1.xxx 1
_EOF_

$test->write(['subdir', 'b2.yyy'], <<_EOF_);
subdir/b2.yyy 1
_EOF_

$test->write(['subdir', 'b3.zzz'], <<_EOF_);
subdir/b3.zzz 1
_EOF_

#
$test->run(targets => ".");

$test->file_matches('f1.out', <<_EOF_);
f1.xxx 1
_EOF_

$test->file_matches('f2.out', <<_EOF_);
f2.yyy 1
_EOF_

$test->file_matches('f3.out', <<_EOF_);
f3.zzz 1
_EOF_

$test->file_matches(['subdir', 'b1.out'], <<_EOF_);
subdir/b1.xxx 1
_EOF_

$test->file_matches(['subdir', 'b2.out'], <<_EOF_);
subdir/b2.yyy 1
_EOF_

$test->file_matches(['subdir', 'b3.out'], <<_EOF_);
subdir/b3.zzz 1
_EOF_

$test->up_to_date(targets => ".");

#
$test->write('f1.xxx', <<_EOF_);
f1.xxx 2
_EOF_

$test->write('f2.yyy', <<_EOF_);
f2.yyy 2
_EOF_

$test->write('f3.zzz', <<_EOF_);
f3.zzz 2
_EOF_

$test->write(['subdir', 'b1.xxx'], <<_EOF_);
subdir/b1.xxx 2
_EOF_

$test->write(['subdir', 'b2.yyy'], <<_EOF_);
subdir/b2.yyy 2
_EOF_

$test->write(['subdir', 'b3.zzz'], <<_EOF_);
subdir/b3.zzz 2
_EOF_

$test->adjust_consign_times('.');
$test->adjust_consign_times('subdir');

#
$test->run(targets => ".");

$test->file_matches('f1.out', <<_EOF_);
f1.xxx 1
_EOF_

$test->file_matches('f2.out', <<_EOF_);
f2.yyy 1
_EOF_

$test->file_matches('f3.out', <<_EOF_);
f3.zzz 2
_EOF_

$test->file_matches(['subdir', 'b1.out'], <<_EOF_);
subdir/b1.xxx 1
_EOF_

$test->file_matches(['subdir', 'b2.out'], <<_EOF_);
subdir/b2.yyy 2
_EOF_

$test->file_matches(['subdir', 'b3.out'], <<_EOF_);
subdir/b3.zzz 2
_EOF_

$test->up_to_date(targets => ".");

#
$test->pass;
__END__
