#! /usr/bin/env perl
#
#	Create subdirectories work, foo, foo/xxx, bar, and bar/yyy.
#	Construct file specifies a CPPPATH of .:zzz, LIBPATH of yyy:.,
#	a Program that builds a single .c file, and Commands that print
#	%CCCOM and %LINKCOM to output files.  In the work directory,
#	invoke cons -R foo -R work -R bar.  Examine the CCCOM and LINKCOM
#	output to make sure that the lists of directories specified via
#	-I and -L flags properly exclude the work subdirectory in which
#	we executed, the non-existent local zzz and yyy directories,
#	and non-existent repository directories.  Remove targets and
#	make zzz and yyy directories in the work directory.  Re-run
#	Cons.  Examine the CCCOM and LINKCOM output to make sure that
#	the now-existing zzz and yyy directories show up.
#
#	NOTE:  THIS TEST EXAMINES THE ACTIONS USED TO BUILD FILES.
#

# $Id: t0141.t,v 1.12 2001/05/25 14:50:14 knight Exp $

# Copyright (c) 1996-2001 Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

use Test::Cmd::Cons qw($_rules $_exe);

$test = Test::Cmd::Cons->new('string' => '-R, -I/-L directories',
			     match_sub => \&Test::Cmd::diff_regex);

$test->subdir('foo', ['foo', 'zzz'], 'bar', ['bar', 'yyy'], 'work');

#
$foo_exe = "foo$_exe";
$workpath_foo = $test->workpath('foo');
$workpath_foo_zzz = $test->workpath('foo', 'zzz');
$workpath_bar = $test->workpath('bar');
$workpath_bar_yyy = $test->workpath('bar', 'yyy');
$workpath_work = $test->workpath('work');
$work_CCCOM_out = $test->catfile('work', 'CCCOM.out');
$work_LINKCOM_out = $test->catfile('work', 'LINKCOM.out');

#
$test->write(['work', 'Construct'], <<_EOF_);
DefaultRules(RuleSet("$_rules"));

\%env_hash = ( ${\$test->cons_env} );
\$env_hash{CPPPATH} = ['.', 'zzz'];
\$env_hash{LIBPATH} = ['yyy', '.'];
\$env = new cons ( \%env_hash );
Program \$env '$foo_exe', 'foo.c';
Command \$env 'CCCOM.out', '$foo_exe', qq(
	\Q$^X\E -e "print '\\Q\%CCCOM\\E', \\\\"\\\\n\\\\"" > %>
);
Command \$env 'LINKCOM.out', '$foo_exe', qq(
	\Q$^X\E -e "print '\\Q\%LINKCOM\\E', \\\\"\\\\n\\\\"" > %>
);
_EOF_

$test->write(['work', 'foo.c'], <<'_EOF_');
main()
{
	printf("work/foo.c");
}
_EOF_

#
$test->run('chdir' => 'work', flags => "-R $workpath_foo -R $workpath_work -R $workpath_bar", targets => ".");

#
$expect = <<_EOF_;
.
$workpath_foo
$workpath_bar
$workpath_foo_zzz
_EOF_

$test->read(\$result, $work_CCCOM_out);
$result = join("\n", grep(s#^[-/]I(.*)#$1#, split(/\s/, $result))) . "\n";

$test->fail($expect ne $result => sub {print STDERR "Expected -I list =====\n", $expect, "Actual -I list =====\n", $result});

#
$expect = <<_EOF_;
$workpath_bar_yyy
.
$workpath_foo
$workpath_bar
_EOF_

$test->read(\$result, $work_LINKCOM_out);
$result = join("\n", grep(s#^[-/]L(?:IBPATH:)?(.*)#$1#, split(/\s/, $result))) . "\n";

$test->fail($expect ne $result => sub {print STDERR "Expected -L list =====\n", $expect, "Actual -L list =====\n", $result});

#
$test->run('chdir' => 'work', flags => '-r', targets => ".");

$test->subdir(['work', 'zzz'], ['work', 'yyy']);

#
$test->run('chdir' => 'work', flags => "-R $workpath_foo -R $workpath_work -R $workpath_bar", targets => ".");

#
$expect = <<_EOF_;
.
$workpath_foo
$workpath_bar
zzz
$workpath_foo_zzz
_EOF_

$test->read(\$result, $work_CCCOM_out);
$result = join("\n", grep(s#^[-/]I(.*)#$1#, split(/\s/, $result))) . "\n";

$test->fail($expect ne $result => sub {print STDERR "Expected -I list =====\n", $expect, "Actual -I list =====\n", $result});

#
$expect = <<_EOF_;
yyy
$workpath_bar_yyy
.
$workpath_foo
$workpath_bar
_EOF_

$test->read(\$result, $work_LINKCOM_out);
$result = join("\n", grep(s#^[-/]L(?:IBPATH:)?(.*)#$1#, split(/\s/, $result))) . "\n";

$test->fail($expect ne $result => sub {print STDERR "Expected -L list =====\n", $expect, "Actual -L list =====\n", $result});

#
$test->pass;
__END__
